/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2006, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.balalaika;

import org.eclipse.birt.report.engine.api.*;
import org.eclipse.birt.core.framework.Platform;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Core BIRT service component. Requires to step through the lifecycle
 * before it can be used:
 * <ul>
 * <li>create: constructs a BIRT engine and platform
 * <li>start:
 * <li>stop: 
 * <li>destroy: destroy engine and shutdown platform
 *
 * @see org.eclipse.birt.report.engine.api.IReportEngine#destroy() 
 * @see org.eclipse.birt.core.framework.Platform#shutdown()
 * 
 * @author Heiko.Braun <heiko.braun@jboss.com>
 */
public class BirtService
{
   private static final Log log = LogFactory.getLog(BirtService.class);
   
   private IReportEngine engine;
   private IntegrationConfig iConfig;

   private enum State  {NONE, CREATED, STARTED, STOPPED, DESTROYED};
   private State currentState = State.NONE;

   public BirtService(IntegrationConfig iConfig)
   {
      this.iConfig = iConfig;
   }

   public void create()
   {
      this.engine = BirtEngineFactory.newInstance(iConfig);
      log.info("Service created: " +this.engine);
      currentState = State.CREATED;
   }

   public void start()
   {
      if(currentState!=State.CREATED)
         throw new IllegalStateException("Please call create lifecylce before starting the service");
      
      currentState = State.STARTED;
   }

   public void stop()
   {
      if(currentState!=State.STARTED)
         throw new IllegalStateException("Please call start lifecylce before stopping the service");

      currentState = State.STOPPED;
   }

   public void destroy()
   {
      if(engine!=null)      
         engine.destroy();
      Platform.shutdown();
      log.info("Service destroyed");
      currentState = State.DESTROYED;
   }

   public String render(RenderMetaData metaData)
   {
      if( (currentState==State.CREATED || currentState==State.STARTED) == false)
         throw new IllegalStateException("Cannot render in state " + currentState);
      
      String outputFileName = null;

      log.debug("Render " + metaData);

      try
      {
         //Open a report design
         IReportRunnable design = engine.openReportDesign(iConfig.getReportDir()+metaData.getReportName());

         //Create runRenderTask to run and render the report,
         IRunAndRenderTask runRenderTask = engine.createRunAndRenderTask(design);
         //Set parent classloader for engine
         runRenderTask.getAppContext().put(EngineConstants.APPCONTEXT_CLASSLOADER_KEY, metaData.getClassloader());
         runRenderTask.setParameterValues(metaData.getParameters());
         
         // ------------------
         
         IRenderOption options = new RenderOption();
         switch(metaData.getFormat())
         {
            case HTML:
               options.setOutputFormat("html");
               outputFileName = extactReportName(metaData.getReportName())+".html";
               break;
            case PDF:
               options.setOutputFormat("pdf");
               outputFileName = extactReportName(metaData.getReportName())+".pdf";
               break;
         }
         options.setOutputFileName(iConfig.getOutputDir() + outputFileName);

         // ------------------

         if( options.getOutputFormat().equalsIgnoreCase("html"))
         {
            HTMLRenderOption htmlOptions = new HTMLRenderOption( options);
            htmlOptions.setImageHandler(new HTMLServerImageHandler());
            htmlOptions.setImageDirectory(iConfig.getImageDirectory());
            htmlOptions.setBaseImageURL(iConfig.getImageBaseUrl());           
            htmlOptions.setHtmlPagination(false);
            htmlOptions.setHtmlRtLFlag(false);
            //Setting this to true removes html and body tags
            htmlOptions.setEmbeddable(false);
            runRenderTask.setRenderOption(htmlOptions);
         }
         else if( options.getOutputFormat().equalsIgnoreCase("pdf") ){

            PDFRenderOption pdfOptions = new PDFRenderOption( options );
            pdfOptions.setOption( IPDFRenderOption.FIT_TO_PAGE, new Boolean(true) );
            pdfOptions.setOption( IPDFRenderOption.PAGEBREAK_PAGINATION_ONLY, new Boolean(true) );
            runRenderTask.setRenderOption(pdfOptions);
         }
         
         // ------------------
         if(runRenderTask.getRenderOption()==null)
            runRenderTask.setRenderOption(options);

         runRenderTask.run();
         runRenderTask.close();
      }
      catch (EngineException e)
      {
         throw new RuntimeException("Failed to render report", e);
      }

      return outputFileName;
   }

   public IntegrationConfig getIConfig()
   {
      return iConfig;
   }

   private static String extactReportName(String reportFile)
   {
      return reportFile.substring( 0, reportFile.indexOf("."));
   }

   public State getCurrentState()
   {
      return currentState;
   }
}
